@@ -4,9 +4,7 @@ use strict;
 use warnings;
 use Data::Dumper;
 use base qw(Wx::App);
-use Wx qw(:everything);
-use Wx::Event qw(EVT_COMBOBOX EVT_TEXT EVT_TEXT_ENTER EVT_CLOSE);
-
+use Wx::Perl::DbLinker::Wxform;
 use Gtk2::Ex::DbLinker::DbcDataManager;
 use  Log::Log4perl;
 
