@@ -260,6 +260,10 @@ sub _setup_grid {
 			    } else {
 					
 				$self->SetCellRenderer($r, $col, $field->{editor} );
+				if ($field->{renderer_function}) {
+					my $coderef = $field->{renderer_function};
+					$self->SetCellValue($r, $col, &$coderef($r, $col, $self->GetTable));
+				}
 			
 				 
 			}
@@ -1268,6 +1272,32 @@ C<name> / name of the field to display.
 
 C<renderer> / one of "text combo toggle hidden".
 
+=item *
+
+C<renderer_function> for text render, a coderef can be pass to set the cell value. The function will received the row and column number and the base table object.
+For example, this method
+
+		sub display_url {
+			my ($self, $row, $col, $table) =@_;
+	 		my $column_no = $self->{foo}->colnumber_from_name( "fooid" ); # the column where a value is taken from
+			my $key_value = $table->GetValue($row, $column_no );
+			my $result="";
+			if ($key_value) {
+				$result = "http://somewhere/on_the_internet/record.cgi?&fooid=" . $key_value;
+			} 
+	 		return $result;
+		}
+
+It can be passed to the Wxdatasheet constructor with
+
+ 		{ fields =>  [{name=>"bar", renderer=>"text"},
+			{name=>"fooid"}, 
+			{name=>"url", renderer=>"text", renderer_function => sub {display_url ($self, @_);}},
+		             ], 
+		  data_manager =>  ...
+		  }
+		 
+
 =back
 
 if the renderer is a combo (Wx::ComboBox or Wx::ListBox) the following key / values are needed in the same hash reference:
