@@ -49,11 +49,11 @@ my %setter = (
 
 my %getter = (
 	 'Wx::TextCtrl' => sub {my ($self, $w, $id) = @_; return $w->GetValue;},
-	'Wx::ToggleButton' => sub {my ($self, $w, $id) = @_; return $w->get_active;},
+	'Wx::ToggleButton' => sub {my ($self, $w, $id) = @_; return $w->GetValue;},
 	'Wx::ComboBox' => ,  sub { my ($self, $w, $id) = @_; return $self->_get_combobox_selectedvalue($id)},
 	'Wx::ListBox' => ,  sub { my ($self, $w, $id) = @_; return $self->_get_combobox_selectedvalue($id)},
-	'Wx::CheckBox' => sub { my ($self,  $w, $id) = @_; return $w->get_active;},
-	'Wx::SpinButton' => sub {my ($self, $w, $id) = @_; return $w->get_active;},
+	'Wx::CheckBox' => sub { my ($self,  $w, $id) = @_; return $w->IsChecked;},
+	'Wx::SpinButton' => sub {my ($self, $w, $id) = @_; return $w->IsChecked;},
 	'Wx::ListCtrl' =>   sub{ my ($self, $w, $id) = @_; $self->_get_combobox_selectedvalue($id);},
 	
 );
@@ -101,7 +101,7 @@ sub new {
  	$self->{dates_formatters} = {};
 
  	$self->{pos2del} = [];
-
+	$self->{inserting} = 0;
 	return $self;
   
 } #new
@@ -178,39 +178,43 @@ sub add_combo_values {
 	my ($self, $w, $aref, $name)=@_;
 	# $self->{log}->debug( "build_list $name\n");
 	my $wref = ref $w;
-	confess ("only Wx::ComboBox supported by add_combo_values") unless ($wref eq "Wx::ComboBox");
+	my @supp_class = (qw/Wx::ComboBox Wx::ListBox/);
+	my %supported = map {$_ => 1} @supp_class;
+	confess ("only ". join(" ", @supp_class)  . " supported by add_combo_values") unless ($supported{$wref});
 
 	my $size=0;
 	if (defined $aref->[0]){$size = scalar (@{$aref->[0]});} else {$aref=[];}
 	# die ($size);
-	my (@row_val,$i, %ids, %idpos);
+	my (@row_val,$col_i, %ids, %idpos);
 	
 	# my $lst = Gtk2::ListStore->new(@{ $self->{lists}->{$name}->{glib} });
-	my $pos=0;
+	my $i_pos=0;
 	 foreach my $row ( @$aref ) {
 		 # push @row_val, $lst->append; 
 		 #  print "Data: $row->[0]\n";
 		 my $display;
 		
-		 for ($i=0; $i<$size; $i++){ 
-			 if ($i == 0) {
+		 for ($col_i=0; $col_i<$size; $col_i++){ 
+			 if ($col_i == 0) {
 				 #push @ids, $row->[$i];
-			    $self->{log}->debug("add_combo_values val:" . $row->[$i] . " index: " . $pos);
-			     $ids{$i} = $row->[$i];
-			     $idpos{$row->[$i]}= $pos;
+				 # $self->{log}->debug("add_combo_values val:" . $row->[$col_i] . " index: " . $i_pos);
+			     $ids{$i_pos} = $row->[$col_i];
+			     $idpos{$row->[$col_i]}= $i_pos;
 
 
 			 } else {
 			 # push @model, $i, $row->[$i];
-			  	$display .= $row->[$i] . " ";
+			  	$display .= $row->[$col_i] . " ";
 		         }
 		 }
 		 #push @row_val, $display; # if ($pos == $lastcol); 
 		 $w->Append($display);
-		 $pos++;
+		 $i_pos++;
 	}
 	        $self->{datawidgetsValue}->{ $name } = \%ids;
        		$self->{datawidgetsPos}->{ $name } = \%idpos;
+		#$self->{log}->debug("dwV: \n" . Dumper(%ids));
+		#$self->{log}->debug("dvP: \n" . Dumper(%idpos));
 
 }
 
@@ -469,7 +473,7 @@ sub _display_data {
    my $first = ($pos < 0 ? 0 : 1);
   
    $self->_set_record_status_label;
-   $self->_set_rs_range($first);
+   $self->_set_rs_range($first,  $self->{dman}->row_count);
    $self->{on_current}() if ($self->{on_current});
    $self->{painting}=0;
    $self->{changed}=0;
@@ -481,6 +485,7 @@ sub undo{
 	my $self = shift;
 	 $self->{log}->debug("undo clicked");
 	 $self->{changed}=0;
+	  $self->{inserting} = 0;
 	 $self->{pos2del}= [];
 	 $self->_display_data( $self->{pos}  );
 
@@ -495,15 +500,23 @@ sub insert {
 	my $self = shift;
 	$self->{log}->debug("insert");
 	# my $row = $self->{data}[0]->new;
-
+	$self->{inserting} = 1;
 	#$self->{pos} = $self->{count} + 1;
 	#afficher des champs vides
-	$self->_display_data(-1);
-	#data_manager->new_row is called when apply is cliked
+	#$self->_display_data(-1);
+	my $new_pos = $self->{dman}->row_count;
+	my $first = ($new_pos > 0 ? 1 : 0);
+	#data_manager->new_row is called when apply is cliked / but defaults value are not displayed then
+	$self->{dman}->new_row;
+	$self->{log}->debug("insert : row count is : ", $self->{dman}->row_count, " new pos is : ", $new_pos);
+	# SqlADM and DBIDM ->new_row does not change row_count (are the others DM similar ?)
+	$self->_display_data($new_pos);
 	   if ($self->{rec_spinner}){
 		  #	my $last = $self->{dman}->row_count;
 		  #$self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
-        	$self->{rec_spinner}->SetValue(0);
+	         $self->{rec_spinner}->SetRange( $first, $new_pos+1 );
+		 #ne pas appler _rs_on_changed ici
+        	$self->{rec_spinner}->SetValue($new_pos+1);
 		#$self->{rec_spinner}->signal_handler_unblock( $self->{rs_value_changed_signal} );
 		 
     
@@ -533,7 +546,7 @@ sub apply{
 	if ($self->{pos}<0){
 		#my $class =  $self->{class};
 		#$row = $class->new;
-		 $self->{dman}->new_row;
+		#$self->{dman}->new_row;
 			#push @{$self->{data}}, $row;
 			 # $self->{count} ++;
 		 $self->{log}->debug("New row");
@@ -544,14 +557,23 @@ sub apply{
 	# deleting a (or some) record	
 	for my $p (@{$self->{pos2del}}){
 		$self->{dman}->set_row_pos($p);
+
 		$self->{dman}->delete;
+
 	}
 	$self->{log}->debug("items in pos2del: " . scalar @{ $self->{pos2del} } );
 	if (scalar @{$self->{pos2del}}){
 		$self->{pos2del} = [];
 		$self->{changed} = 0;
 		# $self->set_record_status_label;
-		$self->{rec_spinner}->SetValue(1) if ($self->{rec_spinner});
+		#$self->{rec_spinner}->SetValue(1) if ($self->{rec_spinner});
+		my $last = $self->{dman}->row_count;
+		if ($last > 0) {
+			$self->{rec_spinner}->SetValue(1) if ($self->{rec_spinner});
+			$self->_display_data(0);
+		} else {
+			$self->_display_data(-1);
+		}
 		return;
 	}
 	#updating a new or an existing record
@@ -575,7 +597,7 @@ sub apply{
 				$self->{log}->debug("not done because it's a auto incremented pk");
 			} else {
 		
-				$v = ($v eq "" ? undef : $v);
+				$v = (defined $v && $v eq "" ? undef : $v);
 				$self->{log}->debug($id  . ": value undef") unless ( defined $v);	
 				# if ( defined $v && ( $id ~~ @{$self->{dates_formatted}})){
 				if ( defined $v && defined $self->{hdates_formatted}->{$id}){
@@ -615,13 +637,10 @@ sub apply{
 
 
 	for my $pk (@pk) {
-		$self->{log}->debug("Primary Key: " . $pk);
+	
 		my $value = $self->{dman}->get_field($pk);
-		#if (my $ref = eval { $row->can( $pk ) }) {
-			#$value = $row->$ref();
-
-			$pk_val{$pk} = $value;
-		#}
+		$pk_val{$pk} = $value;
+		$self->{log}->debug("Primary Key: " . $pk . " value : " . $value);
 	}
 			#push @pk_val, $id
 		
@@ -629,10 +648,13 @@ sub apply{
 		my $coderef = $self->{after_insert};
 		&$coderef(undef, \%pk_val );
 	}
-	if ($done && $self->{pos}<0){
+	#if ($done && $self->{pos}<0){
+	if ($done && $self->{inserting}){
 		my $last = $self->{dman}->row_count -1;
+		$last = ($last < 0 ? 0 : $last);
 		$self->{log}->debug("last is " . $last );
 		$self->_display_data( $last );
+		$self->{inserting} = 0;
 		 if ($self->{rec_spinner}){
 		  #	my $last = $self->{dman}->row_count;
 		  #$self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
@@ -717,7 +739,7 @@ sub _bind_on_changed {
 
 
 		}
-		$self->{log}->debug("bind  $name $id with self->changed for signal " . $signals{$name} );
+		$self->{log}->debug("bind  $name $id with self->changed");
 		#$w->signal_connect_after( $signals{$name} => sub{ $self->_changed( $id )});
 		die("signal undef for $name") unless($signals{$name});
 
@@ -742,10 +764,6 @@ sub _set_recordspinner {
     # die unless($self->{rec_spinner});
  my $coderef;
     if ( $self->{rec_spinner} ) {
-#	    The return type of the signal_connect() function is a tag that identifies your callback function. 
-#	    You may have as many callbacks per signal and per object as you need, and each will be executed in turn, 
-#	    in the order they were attached. 
-	    #$coderef  = $self->{rec_spinner}->signal_connect_after(	value_changed => sub {
 	    $coderef =  sub {
 			my $pos = $self->{rec_spinner}->GetValue -1;
 		        $self->{log}->debug("rs_value changed will move to " . $pos);	
@@ -761,7 +779,13 @@ sub _set_recordspinner {
 
 	    
 		EVT_SPINCTRL($self->{rec_spinner}, $self->{rec_spinner}, $coderef);
-		EVT_TEXT($self->{rec_spinner}, $self->{rec_spinner}, $coderef);
+		# With this EVT_TEXT marcro, a change done with ->insert calls the code above
+		# EVT_TEXT($self->{rec_spinner}, $self->{rec_spinner}, $coderef);
+		# from http://docs.wxwidgets.org/trunk/classwx_spin_ctrl.html
+		# if the user modifies the text in the edit part of the spin control directly, the EVT_TEXT is generated, 
+		# like for the wxTextCtrl. When the use enters text into the text area, the text is not validated until the control loses 
+		# focus (e.g. by using the TAB key). The value is then adjusted to the range and a wxSpinEvent sent 
+		# then if the value is different from the last value sent.
 	      $self->{rs_value_changed_signal}= $coderef;
 	      $self->{log}->debug("recordspinner set");
     }
@@ -771,18 +795,13 @@ sub _set_recordspinner {
 
 
 sub _set_rs_range {
-    my ( $self, $first ) = @_;
+    my ( $self, $first, $last ) = @_;
 
     	$self->{log}->debug("set_rs_range  first : " . $first);
 	my $rowcount = $self->{dman}->row_count;
     if ( $self->{rec_spinner} ) {
-	    # $self->{log}->debug("adj lower : ". $min);
-	    # if ($first < $min){ $min = $first;}
-	 #$self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
-	   $self->{rec_spinner}->SetRange( $first, $rowcount );
-	#$self->{rec_spinner}->signal_handler_unblock( $self->{rs_value_changed_signal} );
+	   $self->{rec_spinner}->SetRange( $first, $last );
     }
-    #$self->{rec_count_label}->SetText(" / " . $self->{dman}->row_count);
     #Note that this function will not generate the wxEVT_COMMAND_TEXT_UPDATED event.
     #$self->{rec_count_label}->ChangeValue(" / " . $self->{dman}->row_count);
     $self->{rec_count_label}->SetLabel(" / " . $self->{dman}->row_count);
@@ -827,7 +846,7 @@ sub _set_combo {
 	my ($self, $w, $x, $id) = @_;
 	#return unless (defined $x); the combo must be unselect from the preceding value
 	my $name = $self->{datawidgetsName}->{$id};
-	$self->{log}->debug("set_combo value " . ( defined $x ? $x : " undef") . " widget: ". $name );
+	$self->{log}->debug("set_combo value " . ( defined $x ? $x : " undef") . " widget: ". $name . " id: " . $w->GetName);
 	if ($name eq "Wx::ListCtrl") {
 		#print "value: $x selected : ", $w->GetSelectedItemCount, "\n";
        
@@ -843,7 +862,7 @@ sub _set_combo {
 		#print "keys: ", join(" ", keys (%idpos)), "\n";
 		#print "values: ", join(" ", values (%idpos)), "\n";
 		# print "value: ", $x, "\n";
-		my $i = $idpos{$x};
+		my $i = $idpos{$x} if (defined $x);
 		$self->{log}->debug("_set_combo pos is " . (defined $i ? $i : " undef") );
 		# $w->SetSelection(wxNOT_FOUND);
 		if (defined $i) {
@@ -859,7 +878,7 @@ sub _set_combo {
 
 sub _set_check {
 	my ($self, $w, $x, $id) = @_;
-	$w->set_active( $x );
+	$w->SetValue( $x );
 }
 
 #$comboid is the combo id and the field name in the table that received the value selected in the combo
@@ -894,10 +913,11 @@ sub _get_combobox_selectedvalue  {
 
 		$x = ($pos ==  wxNOT_FOUND ) ? undef : $self->{datawidgetsValue}->{$id}->{$pos};
 		#my $cd = $w->GetClientData($pos) unless ($pos ==  wxNOT_FOUND);
-		$self->{log}->debug("_get_combobox_selectedvalue pos: $pos");
+		#$self->{log}->debug("_get_combobox_selectedvalue pos: $pos");
+		#$self->{log}->debug(Dumper( $self->{datawidgetsValue}->{$id} ));
 		#return $cd->GetData();
 	}
-	$self->{log}->debug("_get_combobox_selectedvalue: found $x ");
+	$self->{log}->debug("_get_combobox_selectedvalue: found " . (defined $x ? $x : " undef"));
 	return $x;
 }
 
